using UnityEngine;
using System.Collections;

public class PerceptionSystem  {
	private Blackboard mWorldBB;
    private Blackboard mAIBB;
    private Actor mActor;
    
    private float mPerceptionRange;
    private ArrayList mVisibleActors;
	private Actor mNearestActor;
	private float mDistanceToNearestActor;
	private Vector3 mDirectionToNearestActor;
	
	private Transform mPlayerTransform;
	private float mDistanceToPlayer;
	private Vector3 mDirectionToPlayer;
	
	private float mSecondsBetweenUpdates;
	private float mLastUpdateTime;

	
	public PerceptionSystem(Blackboard pWorldBB, Blackboard pAIBB, Actor pActor, float pPerceptionRange, float pSecondsBetweenUpdates)
	{
		mWorldBB = pWorldBB;
        mAIBB = pAIBB;
        mActor = pActor;
        mPerceptionRange = pPerceptionRange;
        mSecondsBetweenUpdates = pSecondsBetweenUpdates;
        mLastUpdateTime = 0.0f;
        mVisibleActors = new ArrayList();
        mNearestActor = null;
        mDistanceToNearestActor = 999999999.0f;
        mDirectionToNearestActor = Vector3.zero;
        
        mPlayerTransform = null;
		mDistanceToPlayer = 999999999.0f;;
		mDirectionToPlayer = Vector3.zero;
        
	}
	
	public void Update()
	{
		float elapsed = Time.time - mLastUpdateTime;
		if(elapsed < mSecondsBetweenUpdates)
		{
			return;
		}
		
		//Update Player Data.
		GameObject playerGO = GameObject.FindWithTag ("Player");
		if(playerGO)
		{
			mPlayerTransform = playerGO.transform;
			Vector3 difference = mPlayerTransform.position - mActor.transform.position;
			mDistanceToPlayer = difference.magnitude;
			mDirectionToPlayer = difference.normalized;	
		}
		
		//Update other actors data
		mVisibleActors.Clear();
		Actor lastNearestActor = mNearestActor;
		
		mNearestActor = null;
        mDistanceToNearestActor = 999999999.0f;
        mDirectionToNearestActor = Vector3.zero;
        
        //Loop through all the actors in the world and figure out which are visible and which one is the closest
		ArrayList actors = (ArrayList)mWorldBB.GetData("ActorList");
		for(int i = 0; i < actors.Count; i++)
		{
			Actor a = (Actor)actors[i];
			if(a == mActor)//check to make sure that we aren't looking at ourself
				continue;
			Vector3 diff = a.transform.position - mActor.transform.position;
			
			float dist = diff.magnitude;
			
			//Only process actors within our perception range
			if(dist < mPerceptionRange)
			{
				mVisibleActors.Add(a);
				if(dist < mDistanceToNearestActor)
				{
					mNearestActor = a;
					mDistanceToNearestActor = dist;
					mDirectionToNearestActor = diff.normalized;	
				}
			}
				
		}
		
		//Update the blackboard so other parts of the actor can use the data
		mAIBB.SetData("PD.NearestActorChanged", lastNearestActor == mNearestActor);
		mAIBB.SetData("PD.NearestActor",mNearestActor);
		mAIBB.SetData("PD.DistToNearestActor", mDistanceToNearestActor);
		mAIBB.SetData("PD.DirToNearestActor", mDirectionToNearestActor);
		mAIBB.SetData("PD.VisibleActors",mVisibleActors);
		
		mAIBB.SetData("PD.PlayerTransform",mPlayerTransform);
		mAIBB.SetData("PD.DistanceToPlayer",mDistanceToPlayer);
		mAIBB.SetData("PD.DirectionToPlayer",mDirectionToPlayer);

		
		mLastUpdateTime = Time.time;
		
	}
	
	
}
