using UnityEngine;
using System.Collections;

public class StateMachine {
	protected StateMachineState mCurrState;
	protected Hashtable mStates;
	protected Actor mActor;
	

    public StateMachine(Actor actor)
    {
    	mActor = actor;
        mStates = new Hashtable();
    }

    public virtual StateMachineState SendMessage(string msg)
    {
        string nextState = mCurrState.SendMessage(msg);
        if (nextState != "" && nextState != mCurrState.GetName())
        {
            SetStateByKey(nextState);
        }
        return mCurrState;
    }
    
    public Actor GetActor()
    {
    	return mActor;
    }
	
	public virtual void AddState(StateMachineState state)
	{
		 mStates[state.GetName()] = state;
	}
	

    public virtual string GetStateName()
    {
        return mCurrState.GetName();
    }
    
    
    public virtual StateMachineState GetState()
    {
    	return mCurrState;
    }
    
    public virtual void SetStateByKey(string nextState)
    {
    	if (mStates.ContainsKey(nextState))
        {
        	StateMachineState lastState = mCurrState;		
            StateMachineState newState = (StateMachineState)mStates[nextState];
            if(newState != null && (lastState != newState))
            {
            	if(lastState != null)
            		lastState.OnExitState();
            	newState.OnEnterState();
            	mCurrState = newState;
            }
            
        }
    }
}
