using UnityEngine;
using System.Collections;

public class StateMachineState {
	protected StateMachine mStateMachine;
    protected Hashtable mTransitions;
    protected string mName;
    protected ArrayList mBehaviors;

    public StateMachineState(StateMachine pStateMachine, string pName)
    {
    	mStateMachine = pStateMachine;
        mName = pName;
        mTransitions = new Hashtable();
        mBehaviors = new ArrayList();
    }

    public string GetName()
    {
        return mName;
    }

    public string SendMessage(string msg)
    {
        if (mTransitions.ContainsKey(msg))
        {
            return mTransitions[msg].ToString();
        }
        return "";
    }

    public virtual void AddTransition(string msg, string nextState){
        mTransitions[msg] = nextState;
    }
    
    public virtual void OnEnterState()
    {
    	Debug.Log("Entering State: " + mName);
    	Actor actor = mStateMachine.GetActor();
    	for(int i = 0; i < mBehaviors.Count; i++)
    	{
    		AIBehavior beh = (AIBehavior)mBehaviors[i];
    		beh.SetActor(mStateMachine.GetActor());
    		beh.SetStartTime(Time.time);
    		actor.AddBehavior(beh);
    	}
    }
    
    public virtual void OnExitState()
    {
    	Debug.Log("Exiting State: " + mName);
    	Actor actor = mStateMachine.GetActor();
    	actor.ClearBehaviors();
    }

	public virtual void AddBehavior(AIBehavior beh)
	{
		mBehaviors.Add(beh);
	}
	
	public virtual void ClearBehaviors()
	{
		mBehaviors.Clear();
	}
	
	public virtual void RemoveBehavior(AIBehavior beh)
	{
		mBehaviors.Remove(beh);	
	}
}
